Unit tio;

{ Text Input/Output Unit for TP/BP.

  Released "AS IS" to the public domain - use at your own risk!

  You will be aware that TP/BP does not provide the standard Pascal
  "get" and "put" routines, nor does it provide direct access to the
  current file variable/record/element/component (call it what you
  will). The reasoning given is that "you really don't need this" or
  "this can't be implemented efficiently". Which is all very well,
  but lack of these facilities can make it difficult to port certain
  standard Pascal programs to TP/BP.

  The purpose of this unit is to provide you these facilities, at
  least for text files. The "get" and "put" routines are used in
  exactly the same manner as with standard Pascal. However, direct
  access to the current file variable is less straight-forward and
  requires two separate routines, which are used as follows :

  "ch := f^"     ... becomes ...   "ch := TxtVar(f)"
  "if f^ = ch"   ... becomes ...   "if TxtVar(f) = ch"
  "f^ := ch"     ... becomes ...   "SetTxtVar(f,ch)"

  So, for almost all circumstances, the "TxtVar" and "SetTxtVar"
  routines provide a direct replacement for the "f^" file variable.
  However, it should be noted, that if you use the "TxtVar" and "get"
  routines to read the CR/LF end-of-line marker, this will NOT be
  translated into a "space" character. This is in keeping with the
  TP/BP "read" procedure, so as to maintain the equivalence between
  "read(f,ch)" and "ch := f^; get(f)" or "ch := TxtVar(f); get(f)".

  Joe da Silva}

INTERFACE

Uses
   dos;

Function TxtVar (var f : text) : char;

Procedure SetTxtVar (var f : text; ch : char);

Procedure get (var f : text);

Procedure put (var f : text);

IMPLEMENTATION

Type
   filefunc = function (var f : text) : integer;

Function TxtVar (var f : text) : char;
const
   EndMarker = ^Z;
begin {TxtVar}
   with TextRec(f) do
      if BufPos < BufEnd
      then
         TxtVar := char(mem[seg(BufPtr^):ofs(BufPtr^)+BufPos])
      else
         TxtVar := EndMarker
end; {TxtVar}

Procedure SetTxtVar (var f : text; ch : char);
begin {SetTxtVar}
   with TextRec(f) do
      if BufPos < BufSize
      then
         mem[seg(BufPtr^):ofs(BufPtr^)+BufPos] := byte(ch)
end; {SetTxtVar}

Procedure get (var f : text);
begin {get}
   with TextRec(f) do
      begin {with}
         if BufPos < BufEnd
         then
            inc(BufPos);
         if BufPos >= BufEnd
         then
            InOutRes := filefunc(InOutFunc)(f)
      end {with}
end; {get}

Procedure put (var f : text);
begin {put}
   with TextRec(f) do
      begin {with}
         if BufPos < BufSize
         then
            inc(BufPos);
         if BufPos >= BufSize
         then
            InOutRes := filefunc(InOutFunc)(f)
      end {with}
end; {put}

END. {tio}
